/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#include <QGraphicsWidget>
#include <QGraphicsGridLayout>
#include <QGraphicsTextItem>
#include <QGraphicsProxyWidget>
#include <KMessageBox>
#include <KRun>
#include <QDebug>
//////////////////////////////////////
#include "device.h"
#include "settings.h"
#include "dia_change_nick_name.h"
//////////////////////////////////////

/////////////////////////////////////////////////////////////
/////         																					/////
/////     Class Device																	/////
/////         																					/////
/////////////////////////////////////////////////////////////
Device::Device( const QString& dev, const QString& mt, SettingsStructure *set) : QObject (NULL) , check_state(this)
{
		settings=set;
		current_command="";
				
		set_max_space(1);
		set_free_space(0);
		set_type(HardDisk);
		set_managed(!FALSE);
		set_dev_name(dev);
		set_mount_point(mt);
		set_nick_name(mt);
		set_state(Unmounted);
		set_show_free_space(TRUE);
		set_sudo(FALSE);
		set_connection_state(Off);
		
		command_timeout.setSingleShot(TRUE);		
		
		command = new KProcess(this);
		command->setOutputChannelMode(KProcess::SeparateChannels);
		command->setNextOpenMode(QIODevice::ReadWrite);
		connect(command,SIGNAL(finished( int, QProcess::ExitStatus )) , this ,SLOT(command_finished( int, QProcess::ExitStatus )));
		connect(command,SIGNAL(error( QProcess::ProcessError )) , this ,SLOT(command_failed( QProcess::ProcessError )));
//  		connect(command,SIGNAL(readyReadStandardOutput()) , this ,SLOT(enter_input()));
		connect(&check_state,SIGNAL(timeout()),this,SLOT(update_connection_state()));
		connect(&command_timeout,SIGNAL(timeout()),this,SLOT(command_timedout()));
		
}

//////////////////////////////////
/////   Interactions
////////////////////////////////
void Device::launch_file_manager()
{
		KRun *proc = new KRun (KUrl(get_mount_point()),NULL,(mode_t)0,TRUE,TRUE);
}

void Device::do_mount()
{
				
		if (state==Mounting)
		{
			stop_mounting();
			command->terminate();
			state=Unmounted;
			return;
		}
		else if (state==Unmounting)
		{
			stop_mounting();
			command->terminate();
			state=Mounted;
			return;
		}
		else 
		{
			if (state==Mounted)
				state=Unmounting;
			else
				state=Mounting;
			start_mounting();
		}
		
		current_command="mount";
		
		QString mnt="mount";
		QString umnt="umount";
		QStringList args;				
		
		if (((get_type()==FUSE_FS) || (get_type()==SSH_FS)))
		{		
			mnt = "mount";
			umnt="fusermount";
			if (state==Unmounting)
				args << "-u";			
		}
		
		if (is_sudo())
		{
			if (state == Mounting)
				args << "-t" << mnt << get_mount_point();
			else
				args << "-t"<< umnt << get_mount_point();				
		}
		else
		{
			args << get_mount_point();
			if (state == Mounting)
				command->setProgram(mnt,args);
			else
				command->setProgram(umnt,args);
			
		}
		//qDebug() << command->program() << " - " << state;
		command->start();		
}

void Device::do_load()
{	
		if (load_state==Connecting)
		{
			stop_loading();
			command->terminate();
			load_state=Off;
			return;
		}
		else if (load_state==Disconnecting)
		{
			stop_loading();
			command->terminate();
			load_state=Ready;
			emit SIG_refresh_mtab();
			return;
		}
		else 
		{
			if ((load_state==Ready) || (load_state==On)) 
				load_state=Disconnecting;
			else if (load_state==Off)
				load_state=Connecting;
			start_loading();
		}
		
		QStringList args;		
		current_command="load";
		
		QString cur_com;
		if (load_state==Connecting)
			cur_com=structure.load_command;
		else if (load_state==Disconnecting) 
			cur_com=structure.eject_command;		
		
		cur_com=cur_com.replace("%1",get_dev_name());
		cur_com=cur_com.replace("%2",get_ip_address());
		
		
		QStringList command_line = cur_com.split(" ");
		
		command->setProgram(command_line);		
		command->start();
}


void Device::command_timedout()
{
	command_failed (QProcess::Timedout);	
}


void	Device::command_failed							(QProcess::ProcessError err)
{
		QString S;
		for (int n=0;n<command->program().count();n++)
			S+=command->program().at(n)+" ";
		
		switch (err)
		{
			case QProcess::Timedout :
				KMessageBox::detailedError(NULL, QString("The requested command ('%1') didn't complete within the required time (%2 s)").arg(S).arg(settings->preferences.mount_timeout), command->readAllStandardError() , "Command timedout" );	
				break;
			case QProcess::FailedToStart :
				KMessageBox::detailedError(NULL, QString("The requested command ('%1') didn't start").arg(S), command->readAllStandardError() , "Command failed to start" );	
// 				qDebug() << "program didn't start";
				break;
				
			case QProcess::UnknownError :
				KMessageBox::detailedError(NULL, QString("The requested command ('%1') ran into an unknown error. Sorry.").arg(S), command->readAllStandardError() , "Command failed for some reason" );	
// 				qDebug() << " unknown error";
				break;
				
			default :
// 				qDebug() << "error ??";
				break;
		}
		
		
		if (current_command=="mount")
			stop_mounting();
		else
			stop_loading();
		
		emit SIG_refresh_mtab();
		current_command="";	
}

void	Device::command_finished (int exitCode, QProcess::ExitStatus exitStatus)
{
		if (exitStatus != QProcess::NormalExit)
		{
			return;
		}
		
		if (exitCode != 0)
		{
			if (current_command=="mount")
				KMessageBox::detailedError(NULL, "Cannot un/mount the Device, maybe you don't have the necessary permissions", command->readAllStandardError() , "Mount Error" );
			else if (current_command=="load")
				KMessageBox::detailedError(NULL, "Cannot load/eject or dis/connect the Device, maybe this device cannot be affected by the action", command->readAllStandardError() , "Eject Error" );
		}
		
		
		if (current_command=="mount")
			stop_mounting();
		
		emit SIG_refresh_mtab();
		current_command="";
	
}

void Device::update_connection_state()
{
	
	ConnectState new_st = check_connection_state();
	
	if (((load_state==Connecting) && (new_st==Ready)) || ((load_state==Disconnecting) && (new_st==Off)))
		stop_loading();	
	
	set_connection_state(new_st);				
}


ConnectState Device::check_connection_state() const
{
	if (is_remote() && !get_ip_address().isEmpty())
	{
		QStringList prog;
		prog << "ping" << get_ip_address() << "-c1";
		if (KProcess::execute(prog,settings->preferences.ping_timeout) == 0)
			return Ready;
		else
			return Off;			
	}
	return Unavailable;
}

void Device::start_mounting()
{
	command_timeout.start(settings->preferences.mount_timeout*1000);
	emit SIG_animation_changed();
}


void Device::stop_mounting()
{
	command_timeout.stop();
	state=Unmounted;
	
	emit SIG_animation_changed();
}

bool Device::is_animated()
{
	return (state==Mounting) || (state==Unmounting) || (load_state==Connecting) || (load_state==Disconnecting);
}

void Device::start_loading()
{
	command_timeout.start(settings->preferences.mount_timeout*1000);
	check_state.start(1000);
	
	emit SIG_animation_changed();
}

void Device::stop_loading()
{
	command_timeout.stop();
	check_state.stop();
	load_state=Off;
	
	emit SIG_animation_changed();
}

////////////////////////////////////////////////////
////   Data handling and syncing
////////////////////////////////////////////////////
void Device::some_data_changed()
{ 
	// Make sure that everything is synced properly
	set_info(get_info());
	refresh_free_space_text();	
}

void Device::refresh_free_space_text()
{
	if (!settings->initialized)
		return;
	
	float fs = 1.0*get_free_space();
	float ms = 1.0*get_max_space();
	float us = ms-fs;
	
	Unit u_fs,u_ms,u_us;
	if (settings->preferences.use_best_unit)
	{
		to_best_unit(u_fs,fs);
 		to_best_unit(u_ms,ms);
 		to_best_unit(u_us,us);
	}
	else
	{
		u_fs=settings->preferences.size_unit;
		u_ms=settings->preferences.size_unit;
		u_us=settings->preferences.size_unit;
		to_unit(u_fs,fs);
		to_unit(u_ms,ms);
		to_unit(u_us,us);
	}

	fs_text=QString("%1 %2").arg(fs,0,'f',settings->appearance.digits).arg(UNIT_NAME[u_fs]);
	ms_text=QString("%1 %2").arg(ms,0,'f',settings->appearance.digits).arg(UNIT_NAME[u_ms]);
	us_text=QString("%1 %2").arg(us,0,'f',settings->appearance.digits).arg(UNIT_NAME[u_us]);
	
}

void Device::get_free_space_text(QString &fs,QString &us,QString &ms)
{
	fs=fs_text;
	us=us_text;
	ms=ms_text;
}

QString Device::get_tooltip()
{
	if (!settings->initialized)
		return QString();
	
	QString tooltip=QString("<font color='%1'><table><tr><td colspan='2' align='center'><font color='%9' size='+1'><B><I>%2</I></B></font></td></tr><HR><tr><td>Device is currently</td><td> <font color='%3'>%4</font></td></tr><tr><td>Device name </td><td> <font color='%5'>%6</font></td></tr><tr><td>Mount point</td><td> <font color='%7'>%8</font></td></tr>")
 							.arg(settings->appearance.colors.at(ToolTipText).name())
							.arg(get_nick_name())
							.arg(settings->appearance.colors.at(ToolTipStatus).name())
							.arg(MOUNT_STATE_NAME[get_state()])
							.arg(settings->appearance.colors.at(ToolTipName).name())
 							.arg(get_dev_name())
							.arg(settings->appearance.colors.at(ToolTipMountPoint).name())
 							.arg(get_mount_point())
							.arg(settings->appearance.colors.at(ToolTipNickName).name());
							
	if (is_show_icon_state())
		tooltip+=QString("<tr><td>Network connection </td><td> <font color='%2'>%1</font></td></tr>").arg(CONNECT_STATE_NAME[get_connection_state()]).arg(settings->appearance.colors.at(ToolTipStatus).name());
							
	if (is_show_free_space() && (get_state()==Mounted))
	{
		QString fs,us,ms;
		get_free_space_text(fs,us,ms);
		tooltip+=QString("<tr><td>Free space</td><td><font color='%1'>%2 (%3%)</font></td></tr><tr><td>Used space</td><td><font color='%4'>%5 (%6%)</font></td></tr><tr><td>Device capacity</td><td><font color='%7'>%8</font></td></tr>")
							.arg(settings->appearance.colors.at(ToolTipFreeSpace).name()).arg(fs)						
							.arg(((float)get_free_space())*100.0/get_max_space(),0,'f',settings->appearance.digits)
							.arg(settings->appearance.colors.at(ToolTipUsedSpace).name()).arg(us)						
							.arg(100.0-((float)get_free_space())*100.0/get_max_space(),0,'f',settings->appearance.digits)
							.arg(settings->appearance.colors.at(ToolTipMaxSpace).name()).arg(ms);
	}
	return tooltip+"</table></font>";
}

/////////////////////////////////
void Device::set_type (const DeviceType &t)
{
		structure.type=t;
}

DeviceType Device::get_type() const
{
	return structure.type;
}

void Device::guess_type (const QString &S , DeviceType &type , OperatingSystem &OS , bool &remote) const
{
		// try to guess the OS and physical device types according to the given fstab entries
		if ((S.indexOf("ext")>=0) || (S.indexOf("reiser")>=0))
		{
			type=HardDisk;
			OS=Linux;
			remote=FALSE;
		} 
		else if ((S.indexOf("vfat")>=0) || (S.indexOf("msdos")>=0) || (S.indexOf("ntfs")>=0))
		{
			type=HardDisk;
			OS=Windows;
			remote=FALSE;
		}
		else if ((S.indexOf("iso9")>=0) || (S.indexOf("udf")>=0))
		{
			type=CDRom;
			OS=Undefined;
			remote=FALSE;
		}
		else if (S.indexOf("nfs")>=0)
		{
			type=NetDisk;
			OS=Linux;
			remote=TRUE;
		}
		else if ((S.indexOf("smbfs")>=0) || (S.indexOf("cifs")>=0))
		{
			type=NetDisk;
			OS=Windows;
			remote=TRUE;
		}
		else if (S.indexOf("fuse")>=0)
		{
			type=FUSE_FS;
			OS=Linux;
			remote=FALSE;
			if (get_dev_name().indexOf("sshfs")>=0)
			{
				type=SSH_FS;
				remote=TRUE;
			}
		}
		else
		{
			type=Unknown;
			OS=Undefined;
			remote=FALSE;
		}
	
}

//////////////////// Uncommented plain self-explanatory boring stuff
bool Device::is_remote() const
{
	return ( structure.remote ) ; //&& !get_ip_address().isEmpty() );
}

void Device::set_remote(const bool& B)
{
	structure.remote=B;
	set_show_icon_state(B);
}


QString Device::get_ip_address() const
{
	return structure.ip_address;
}

void Device::set_ip_address(const QString& S)
{
	structure.ip_address=S;
}


ConnectState Device::get_connection_state() const
{
	return load_state;
}


void Device::set_connection_state(const ConnectState& S)
{
	if (load_state!=S)
		{
			if (((load_state!=Connecting) || (S!=Off)) && ((load_state!=Disconnecting) || (S!=Ready)))
				load_state=S;
		}
}


QString Device::get_load_command							() const
{
	return structure.load_command;
}

QString	Device::get_eject_command							() const
{
	return structure.eject_command;
}

void Device::set_load_command(const QString &S)
{
		structure.load_command=S;
}

void Device::set_eject_command(const QString &S)
{
		structure.eject_command=S;
}


QString Device::get_dev_name () const
{
		return structure.dev_name;
}

void Device::set_dev_name						(const QString &S)
{
		structure.dev_name=S;
}

/////////////////////////////////////////
void Device::set_state(const MountState &st)
{
		if (state!=st)
		{
			if (((state!=Mounting) || (st!=Unmounted)) && ((state!=Unmounting) || (st!=Mounted)))
				state=st;
		}
}

////////////////////////////////////////

QString	Device::get_mount_point () const
{
		return structure.mount_point;
}

void Device::set_mount_point					(const QString &S)
{
		structure.mount_point=S;
}

////////////////////////////////////////
QString	Device::get_nick_name () const
{
		return structure.nick_name;
}

void Device::set_nick_name					(const QString &S)
{
		structure.nick_name=S;
}

////////////////////////////////////////
void Device::set_free_space								(const quint64 &f)
{
	free_space=f;
	refresh_free_space_text();
}

quint64	Device::get_free_space								() const
{
	return free_space;
}

////////////////////////////////////////
void	Device::set_max_space									(const quint64 &f)
{
	max_space=f;
	refresh_free_space_text();
}

quint64	Device::get_max_space									() const
{
	return max_space;
}
////////////////////////////////////////

void Device::set_show_free_space (const bool &b)
{
	structure.show_free_space=b;
	//qDebug() << B;
}

bool Device::is_show_free_space() const
{
	return structure.show_free_space;
}

////////////////////////////////////////

void Device::set_show_icon_state(const bool &b )
{
	structure.show_state_icon=b;
}

bool Device::is_show_icon_state() const
{
	return structure.show_state_icon;
}
////////////////////////////////////////

QString	Device::get_icon_name () const
{
	return structure.icon_name;
}

void	Device::set_icon_name									(const QString&s)
{
		structure.icon_name=s;
}
////////////////////////////////////////

QString	Device::get_OS_icon_name							() const
{
		return structure.OS_icon_name;
}

void	Device::set_OS_icon_name							(const QString&s)
{
		structure.OS_icon_name=s;
}
////////////////////////////////////////

void	Device::set_OS												(const OperatingSystem&s)
{
		structure.OS=s;
}

OperatingSystem	Device::get_OS												() const
{
		return structure.OS;
}

////////////////////////////////////////

void Device::set_sudo(const bool &b)
{
		structure.sudo=b;
}

bool Device::is_sudo() const
{
		return structure.sudo;
}

////////////////////////////////
MountState Device::get_state() const
{
		return state;
}

bool	Device::is_managed() const
{
		return structure.managed;
}

void	Device::set_managed(const bool &m)
{
		structure.managed=m;	
}

void Device::stop_managing()
{
		set_managed(FALSE);
}

/////////////////////////////////////////////////////////
DeviceInfo	Device::get_info											() const
{
	return structure;
}

void Device::set_info											(const DeviceInfo &inf)
{
	structure=inf;
	set_nick_name(inf.nick_name);
	set_type(inf.type);
	set_icon_name(inf.icon_name);
	set_OS(inf.OS);
	set_OS_icon_name(inf.OS_icon_name);
 	set_managed(inf.managed);
	set_remote(inf.remote);
	set_ip_address(inf.ip_address);
	set_show_icon_state(inf.show_state_icon);
}

////////////////////////////////////////



Device::~Device()
{	
  		delete command;			
}

#include "device.moc"