/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
//////////////////////
#include "dia_devices.h"

/////////////////////////////////////////////////////////////
/////         																					/////
/////     Class Dia_Devices															/////
/////         																					/////
/////////////////////////////////////////////////////////////

//////////////////// Uncommented plain self-explanatory boring stuff

Dia_Devices::Dia_Devices 	(QWidget *par): QWidget(par)
{
		ui.setupUi(this);
		
		ui.bt_up->setIcon(KIcon("arrow-up"));
		ui.bt_down->setIcon(KIcon("arrow-down"));
		
		connect (ui.bt_up,SIGNAL(clicked(bool)),this,SLOT(bt_clicked()));
		connect (ui.bt_down,SIGNAL(clicked(bool)),this,SLOT(bt_clicked()));
		connect (ui.ls_mt_point,SIGNAL(currentRowChanged(int)),this,SLOT(sync_widgets()));
		connect (ui.cb_type,SIGNAL(currentIndexChanged(int)),this,SLOT(type_changed()));
		connect (ui.cb_OS,SIGNAL(currentIndexChanged(int)),this,SLOT(OS_changed()));
		
		old_index=-1;
 		enable_widgets();
}

void	Dia_Devices::set_value							(const QList< Device* >& dev , const QList<int> &o)
{
	list.clear();
	int n=0;
	while (n<dev.count())
		list << dev.at(n++)->get_info();
	order=o;
	fill_widgets();
}

QList<DeviceInfo>	Dia_Devices::get_value							()
{
	sync_values();
	return list;
}
	

QList< int > Dia_Devices::get_order()
{
	return order;
}


void Dia_Devices::fill_devices()
{
	ui.ls_mt_point->clear();
	int n=0;
	while (n<list.count())
		ui.ls_mt_point->addItem(list.at(order[n++]).mount_point);
}

	
void	Dia_Devices::fill_widgets					()
{
	
	fill_devices();
	
	int n=0;
	while (n<N_DEVICE_TYPE)
		ui.cb_type->addItem(DEVICE_TYPE_NAME[n++]);
	
	n=0;
	while (n<N_OS_TYPE)
		ui.cb_OS->addItem(OS_NAME[n++]);
	
	sync_widgets();
}

void	Dia_Devices::sync_values						()
{
	int n=old_index;
	if ((n<0) || (n>=list.count()))
			return;
	
	list[order[n]].nick_name=ui.txt_nick->text();
	list[order[n]].managed=ui.chk_managed->isChecked();
	list[order[n]].dev_name=ui.txt_dev->text();
	list[order[n]].show_free_space=ui.chk_space->isChecked();
	list[order[n]].type=(DeviceType)ui.cb_type->currentIndex();
	list[order[n]].OS=(OperatingSystem)ui.cb_OS->currentIndex();
	list[order[n]].icon_name=ui.bt_icon->icon();
	list[order[n]].OS_icon_name=ui.bt_icon_OS->icon();
	list[order[n]].sudo=ui.chk_sudo->isChecked();
	list[order[n]].load_command=ui.txt_load->text();
	list[order[n]].eject_command=ui.txt_eject->text();
	list[order[n]].ip_address=ui.txt_ip->text();
	list[order[n]].remote=ui.chk_remote->isChecked();
	list[order[n]].show_state_icon=ui.chk_state->isChecked();
}

void	Dia_Devices::sync_widgets					()
{	
	int n=ui.ls_mt_point->currentRow();
	if ((n<0) || (n>=list.count()))
			return;
	
	sync_values();
	old_index=n;
	DeviceInfo info = list.at(order[n]);
	ui.txt_nick->setText(info.nick_name);
	ui.chk_managed->setChecked(info.managed);
	ui.txt_dev->setText(info.dev_name);
	ui.chk_space->setChecked(info.show_free_space);
	ui.chk_state->setChecked(info.show_state_icon);
	ui.cb_type->blockSignals(TRUE);
	ui.cb_type->setCurrentIndex(info.type);
	ui.cb_type->blockSignals(FALSE);
	ui.cb_OS->blockSignals(TRUE);
	ui.cb_OS->setCurrentIndex(info.OS);
	ui.cb_OS->blockSignals(FALSE);
	ui.bt_icon->setIcon(info.icon_name);
	if (info.OS)
		ui.bt_icon_OS->setIcon(info.OS_icon_name);
	else
	{
		ui.bt_icon_OS->setIcon("");
		ui.bt_icon_OS->setIcon(KIcon());
	}
	ui.chk_sudo->setChecked(info.sudo);
	ui.txt_load->setText(info.load_command);
	ui.txt_eject->setText(info.eject_command);
	ui.txt_ip->setText(info.ip_address);
	ui.chk_remote->setChecked(info.remote);
	ui.chk_state->setChecked(info.show_state_icon);
	enable_widgets();
}

void Dia_Devices::enable_widgets()
{
	bool ok=(ui.ls_mt_point->currentRow()>=0) && (ui.ls_mt_point->currentRow()<list.count());
		
	ui.bt_up->setEnabled(ok && (ui.ls_mt_point->currentRow()>0));
	ui.bt_down->setEnabled(ok && (ui.ls_mt_point->currentRow()<list.count()-1));
	ui.bt_icon->setEnabled(ok);
	ui.txt_nick->setEnabled(ok);
	ui.txt_dev->setEnabled(ok);
	ui.chk_managed->setEnabled(ok);
 	ui.chk_space->setEnabled(ok);
	ui.chk_state->setEnabled(ok);
	ui.cb_type->setEnabled(ok);
	ui.label->setEnabled(ok);
	ui.label_6->setEnabled(ok);
	ui.label_4->setEnabled(ok);
	ui.label_5->setEnabled(ok);
	ui.cb_OS->setEnabled(ok);
	ui.bt_icon_OS->setEnabled(ok && (ui.cb_OS->currentIndex()==Other));
	ui.chk_sudo->setEnabled(ok);
}


void Dia_Devices::bt_clicked()
{
	int n=ui.ls_mt_point->currentRow();
	int dir;
	if ((sender()==ui.bt_up) && (n>0))
		dir=-1;
	else if ((sender()==ui.bt_down) && (n<ui.ls_mt_point->count()-1))
	 dir=1;
	else
		return;
	
	int tmp=order[n];
	order[n]=order[n+dir];
	order[n+dir]=tmp;
	
	ui.ls_mt_point->blockSignals(TRUE);
	fill_devices();
	ui.ls_mt_point->blockSignals(FALSE);
	ui.ls_mt_point->setCurrentRow(n+dir);
}


void Dia_Devices::type_changed()
{
	ui.bt_icon->setIcon(DEVICE_TYPE_ICON[ui.cb_type->currentIndex()]);
	ui.txt_load->setText(DEVICE_LOAD_ACTION[ui.cb_type->currentIndex()]);
	ui.txt_eject->setText(DEVICE_EJECT_ACTION[ui.cb_type->currentIndex()]);
	sync_values();
}

void Dia_Devices::OS_changed()
{
	if (ui.cb_OS->currentIndex())
		ui.bt_icon_OS->setIcon(OS_ICON[ui.cb_OS->currentIndex()]);
	else
	{
		ui.bt_icon_OS->setIcon("");
		ui.bt_icon_OS->setIcon(KIcon());
	}
			
	enable_widgets();
	sync_values();
}
