/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#include <KIcon>
#include <QDebug>
//////////////////////////////
#include "icon_view.h"
#include "icon_widget.h"
#include "text_widget.h"
//////////////////////////////

/////////////////////////////////////////////////////////////
/////         																					/////
/////     Class IconViewWidget													/////
/////         																					/////
/////////////////////////////////////////////////////////////


IconViewWidget::IconViewWidget(QGraphicsWidget *par, QList< Device *>* list, SettingsStructure* set): MountoidView(par,list,set,IconView)
{
	is_moving=FALSE;
}

void IconViewWidget::load_devices()
{

	while (texts.count())
		delete texts.takeFirst();
	while (icons.count())
		delete icons.takeFirst();
	
	for (int n=0;n<devices->count();n++)
	{
		TextWidget *text = new TextWidget(devices->at(n),this);
		text->setParentItem(this);
		texts << text;
		
		IconWidget *icon = new IconWidget(devices->at(n),this);
		icon->setParentItem(this);
		icons << icon;
		
		connect_device(devices->at(n));
	}

	MountoidView::some_data_changed();
}

void IconViewWidget::animation_next_step()
{
	int n=icons.count();
	while (n--)
	{
		IconWidget *ic = icons.at(n);
		if (ic->get_device()->is_animated())
			ic->animation_next_step();
	}
	//qDebug() << "z";
}


void IconViewWidget::some_data_changed(const int& dev)
{
	icons.at(dev)->set_icon(devices->at(dev)->get_icon_name());
	icons.at(dev)->set_icon_OS(devices->at(dev)->get_OS_icon_name());	
	icons.at(dev)->data_changed();
	texts.at(dev)->data_changed();
	
	icons.at(dev)->setVisible (devices->at(dev)->is_managed());
	texts.at(dev)->setVisible (devices->at(dev)->is_managed());
}


void IconViewWidget::set_display_mode(const DisplayMode& M)
{
	
}


void IconViewWidget::icon_moving(IconWidget* I)
{
	is_moving=TRUE;
	int n=0;
	// first we find the number of the moving device
	while ((n<icons.count()) && (icons.at(settings->order[n])!=I))
		n++;
	
	QPointF Ipos = I->pos();
	
	// then we look for the device which is being exchanged with the moving device, if any
	int m=0;
	while ((m<icons.count()) && (!icons.at(settings->order[m])->boundingRect().contains(icons.at(settings->order[m])->mapFromParent(Ipos)) || (m==n)))
		m++;
	
	// found one
	if (m<icons.count())
	{
		// exchange the display rank
		int tmp=settings->order[n];
		settings->order[n]=settings->order[m];
		settings->order[m]=tmp;
		refresh_size();
 		I->setPos(Ipos);
	}
}

////////////////////////////
void IconViewWidget::load_state_changed(Device* )
{
	update();
}


////////////////////////////
QSizeF	IconViewWidget::get_minimum_size()
{
	return QSizeF(2.0*PLASMA_ADJUST+32.0,2.0*PLASMA_ADJUST+32.0);
}


void IconViewWidget::apply_new_size(QSizeF &S)
{
		int n=number_managed();
		if (n==0)
			return;
		
		if (settings->appearance.display_mode==StackedHorizontally)		// stack the devices horizontally
		{		
			// divide the plasmoid width into as many pieces as there are devices
			float	w = (S.width()-PLASMA_ADJUST*2.0) / n;
			
			// take padding into account
			w-=ICON_SPACING;
			float x=PLASMA_ADJUST;
			
			// determine the max height of displayed device 
			// and ask these devices to adjust their geometry to the computed width
			n=devices->count();
			float max_h=0.0;
			while (n--)
			{
				TextWidget *text=texts.at(settings->order[n]);
				IconWidget *icon=icons.at(settings->order[n]);
				if (mapped_device(n)->is_managed())
				{
					text->set_width(w);
					icon->set_width(w);		
					text->setPos(QPointF(x,w+PLASMA_ADJUST));
					icon->setPos(QPointF(x,PLASMA_ADJUST));
					
					float dev_h = icon->boundingRect().height()+text->get_height();
					if (dev_h>max_h)
						max_h=dev_h;
					x+=w+ICON_SPACING;
				}
			}
			// now determine the max height of displayed nicknames
			// so that they can be bottom-aligned (nicer :) )
			n=devices->count();
			float max_text_h=0.0;
			while (n--)
			{
				if (mapped_device(n)->is_managed())
				{
					if (texts.at(settings->order[n])->get_nick_name_height()>max_text_h)
						max_text_h=texts.at(settings->order[n])->get_nick_name_height();
				}
			}
			
			// make the max text height global
			settings->text_height=max_text_h;
			
			// resize to the optimal height
			S.setHeight(max_h+2.0*PLASMA_ADJUST);
		}
		else  // ok, stack'em vertically
		{
			// divide the plasmoid height into as many pieces as there are devices
			float	h = (S.height()-PLASMA_ADJUST*2.0) / n;
			// to keep track of the y-coordinate of the current device
			// will result in the total height of the stacked device, to be adjusted
			float y;
			settings->text_height=0.0;
			do
			{				
				y=PLASMA_ADJUST;				
				
				// ask the devices to adjust their geometry to the computed height
				n=devices->count();
				while (n--)
				{
					TextWidget *text=texts.at(settings->order[n]);
					IconWidget *icon=icons.at(settings->order[n]);
					if (mapped_device(n)->is_managed())
					{
						text->set_width(h-ICON_SPACING);
						icon->set_width(h-ICON_SPACING);		
						text->setPos(QPointF(PLASMA_ADJUST,y+h-ICON_SPACING));
						icon->setPos(QPointF(PLASMA_ADJUST,y));
						y+=icon->boundingRect().height()+text->get_height()+ICON_SPACING;
					}
				}
				h-=1.0;
			}
			while (y>S.height()-PLASMA_ADJUST);
			
			// resize to the optimal width
			S.setWidth(h+2.0*PLASMA_ADJUST);
		}
}

/////////////////////////////

