/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#ifndef ICON_WIDGET_HEADER
#define ICON_WIDGET_HEADER
//////////////////////////
#include <QGraphicsItem>
#include <KIcon>
/////////////////////////
class Device;
class IconViewWidget;
/////////////

// Different states from QIcon
static const QIcon::Mode IconState[]=
{
	QIcon::Disabled,
	QIcon::Active,
	QIcon::Disabled,
	QIcon::Active
};


#define CONNECT_ICON			"mountoid_connect_state"
#define MOUNTING_ICON			"mountoid_i_mounting"
#define UNMOUNTING_ICON		"mountoid_i_unmounting"
#define CONNECT_ICON_C 		15
#define MOUNTING_ICON_C 	20


////////////////////////////////////////////////////////////////////////////
// This class provides the icon GraphicsItem used to display a actual device 
// and interact with the user through the contextual menu
class IconWidget : public QGraphicsItem
{
	public:
										IconWidget						(Device* par ,
																					 IconViewWidget *vpar);
		
		// defines the icon for the device type and the OS
		void						set_icon							(const QString &);
		void						set_icon_OS						(const QString &);
		// takes care of drawing the icon
		void 						paint									(QPainter *painter,
																					const QStyleOptionGraphicsItem *option,
																					QWidget *par);
		// sets the display width of the icon
		void						set_width							(const float &);		
		float						get_width							() const;
		// returns the rect containing the icon
		virtual QRectF	boundingRect					() const;
		// sync data from the device when something changes
		void						data_changed					();
		// slot for animations
		void						animation_next_step		();
		
		// associated parent device
		Device					*get_device						();
		
	protected:
		// calls the view contextual menu
		virtual void 		contextMenuEvent			(QGraphicsSceneContextMenuEvent *ev);
		// to determine when the icon has been clicked or is being moved
		virtual void		mousePressEvent				(QGraphicsSceneMouseEvent *ev);
		virtual void		mouseReleaseEvent			(QGraphicsSceneMouseEvent *ev);
		virtual void		mouseMoveEvent				(QGraphicsSceneMouseEvent *ev);
		
	private:
		// called by data_changed, to sync the content of the tooltip 
		// according to the device informations
		void 						synchronize_tooltip		();
		
		// has thez icon been pressed?
		bool						is_clicked;
		// the actual icons
		KIcon						dev_icon;
		KIcon						OS_icon;
		// the parent device
		Device*					par_dev;
		// the iconview
		IconViewWidget 	*view;
		// the display width 
		float						width;
		
		// store the animation step of the waiting icon
		int							connect_icon_step;
		KIcon						connect_icon;		
		// store the animation step of the mounting icon
		KIcon 					mounting_icon;
		KIcon 					unmounting_icon;
		int							mounting_icon_step;

		
};

#endif