/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#include <KIcon>
#include <QGraphicsProxyWidget>
#include <QGraphicsScene>
#include <QGraphicsSceneContextMenuEvent>
#include <KListWidget>
#include <QPainter>
//////////////////////////////
#include "list_view.h"
#include "icon_widget.h"
#include "text_widget.h"

/////////////////////////////////////////////////////////////
/////         																					/////
/////     Class ItemDelegate
/////         																					/////
/////////////////////////////////////////////////////////////
ItemDelegate::ItemDelegate (ListViewWidget *parent): QItemDelegate(NULL)
{
	listview=parent;
}

#define ICON_SIZE 16

void ItemDelegate::paint(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const
{
  Device *dev = listview->mapped_device(index.row());
	SettingsStructure *set = listview->get_settings();
	
	// draw the used space bar as background of the item
	if (dev->is_show_free_space()  && (dev->get_state()==Mounted))
	{
		float tx=1.0-((float)dev->get_free_space())/dev->get_max_space();
		float rate=tx;
		QColor c0,c1;
		// we want to make an affine coloring of the bar
		// between the three states Empty, HalfFull and Full
		// we first determine in which segment we are
		if (tx<=0.5)	// between Emty and HalfFull
		{
			c0=set->appearance.colors.at(EmptyDevice);
			c1=set->appearance.colors.at(HalfFullDevice);
		}
		else 	// between HalfFull and Full
		{
			c1=set->appearance.colors.at(FullDevice);
			c0=set->appearance.colors.at(HalfFullDevice);
			tx=tx-0.5;
		}
		tx*=2.0;
		float xt = 1.0-tx;
		
		// blend the colors
		QColor br_c;
		br_c.setRedF(c0.redF()*xt + c1.redF()*tx);
		br_c.setGreenF(c0.greenF()*xt + c1.greenF()*tx);
		br_c.setBlueF(c0.blueF()*xt + c1.blueF()*tx );
		br_c.setAlphaF(0.5);
				
		// and finally draw the bar
		QBrush brush(br_c);
		painter->setBrush(brush);
		painter->setPen(Qt::NoPen);
		QRect bar = option.rect;
		bar.setWidth(bar.width()*rate);
		painter->drawRect(bar);
	}
	
	// write the nick name of the device
	painter->setFont(set->appearance.font);
	painter->setPen(set->appearance.colors.at(NickName));
	painter->drawText(option.rect.translated(ICON_SIZE,0),Qt::AlignLeft,dev->get_nick_name());
	
	// and draw the icon of the device
	KIcon icon(dev->get_icon_name());
	
	icon.paint(painter,QRect(option.rect.topLeft(),QSize(ICON_SIZE,ICON_SIZE)), Qt::AlignCenter,IconState[dev->get_state()],QIcon::On);
	
}


QSize ItemDelegate::sizeHint(const QStyleOptionViewItem& option, const QModelIndex& index) const
{
    return QItemDelegate::sizeHint(option, index);
}

/////////////////////////////////////////////////////////////
/////         																					/////
/////     Class ListViewWidget													/////
/////         																					/////
/////////////////////////////////////////////////////////////


////////////////////////////////////////
ListViewWidget::ListViewWidget(QGraphicsWidget *par, QList< Device *>* lst, SettingsStructure* set): MountoidView(par,lst,set,ListView)
{
	list=new KListWidget();
	list->setItemDelegate(new ItemDelegate(this));
	list->setAlternatingRowColors(TRUE);
	list->setSelectionMode(QAbstractItemView::NoSelection);
	list->setContextMenuPolicy(Qt::CustomContextMenu);	
	connect (list,SIGNAL(customContextMenuRequested(QPoint)),this,SLOT(context_menu(QPoint)));
	connect (list,SIGNAL(executed(QListWidgetItem*)),this,SLOT(clicked(QListWidgetItem*)));
	
	widget=par->scene()->addWidget(list);
	widget->setParentItem(this);	
}

void ListViewWidget::load_devices()
{
	list->clear();
	for (int n=0;n<devices->count();n++)
	{
		list->addItem(new QListWidgetItem());
	
		connect_device(devices->at(n));
	}

	MountoidView::some_data_changed();
}

void ListViewWidget::animation_next_step()
{

}


QString ListViewWidget::get_item_text(const int& )
{
	QString space;
	
	return space;
}


void ListViewWidget::some_data_changed(const int& dev)
{
	QString space;
	
	list->item(dev)->setToolTip(mapped_device(dev)->get_tooltip());
	list->item(dev)->setHidden(!mapped_device(dev)->is_managed());
	list->item(dev)->setIcon(KIcon(mapped_device(dev)->get_icon_name()));
	list->item(dev)->setText(mapped_device(dev)->get_nick_name());
}


////////////////////////////
void ListViewWidget::apply_new_size(QSizeF &S)
{
	float max_w=0.0;
	float h=0.0;
	widget->resize(widget->minimumSize());
	
	// we want to get the optimal width
	for (int n=0; n<list->count(); n++)
	{		
		QRectF R = list->visualItemRect(list->item (n));

		if (R.width() > max_w)
			max_w=R.width();
		h+=R.height();
	}
	// with a little margin to avoid scroll bars
	max_w*=1.1;
	h*=1.1;
	
	// locate the listview inside the plasmoid and set the size
	widget->setPos(PLASMA_ADJUST,PLASMA_ADJUST);
	widget->setMinimumWidth(max_w);
	widget->setMinimumWidth(h);
	if(S.width()<max_w+2.0*PLASMA_ADJUST)
		S.setWidth(max_w+2.0*PLASMA_ADJUST);
	if (S.height()<h+2.0*PLASMA_ADJUST)
		S.setHeight(h+2.0*PLASMA_ADJUST);
	
	widget->resize(S.width()-2.0*PLASMA_ADJUST,S.height()-2.0*PLASMA_ADJUST);
}

QSizeF ListViewWidget::get_minimum_size()
{
	QSizeF S=widget->minimumSize();
	return QSizeF(2.0*PLASMA_ADJUST+S.width(),2.0*PLASMA_ADJUST+S.height());
}
/////////////////////////////


void ListViewWidget::set_display_mode(const DisplayMode& M)
{
	
}

void ListViewWidget::context_menu(const QPoint& P)
{
	QGraphicsSceneContextMenuEvent *ev= new QGraphicsSceneContextMenuEvent();
	ev->setScreenPos(list->mapToGlobal(P));
	int row = list->row(list->itemAt(P));
	if ((row>=0) && (row<devices->count()))
		MountoidView::context_menu(ev,mapped_device(row));
	else
		MountoidView::context_menu(ev,NULL);
}


void ListViewWidget::clicked(QListWidgetItem* item)
{
	int row = list->row(item);
	if ((row>=0) && (row<devices->count()))
		icon_clicked(mapped_device(row));
}

