/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <KLocale>
#include <QTimer>
#include <QFile>
#include <QSettings>
#include <KConfigDialog>
#include <KMessageBox>
#include <QGraphicsScene>
#include <QGraphicsSceneResizeEvent>
#include <QGraphicsSceneMouseEvent>
#include <plasma/theme.h>
#include <sys/statvfs.h>
#include <math.h>
///////////////////////////////////////
#include "mountoid.h"
#include "device.h"
#include "dia_devices.h"
#include "dia_preferences.h"
#include "dia_appearance.h"
#include "icon_view.h"
#include "list_view.h"
///////////////////////

/////////////////////////////////////////////////////////////
/////         																					/////
/////     General functions															/////
/////         																					/////
/////////////////////////////////////////////////////////////
void to_best_unit (Unit &unit , float &size)
{
	int n=0;
	while ((n<N_UNITS) && (floor(size/1024.0) > 0))
	{
		size/=1024.0;
		n++;
	}
	unit=(Unit)n;
	
}

void to_unit (Unit unit , float &size)
{
	int n=0;
	while (n<unit)
	{
		size/=1024.0;
		n++;
	}	
}

/////////////////////////////////////////////////////////////
/////         																					/////
/////     Class Mountoid																/////
/////         																					/////
/////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////
/////     Initialization
/////////////////////////////////////////////////////////////

Mountoid::Mountoid(QObject *parent, const QVariantList &args)
        : Plasma::Applet(parent, args)
{
		settings.initialized=FALSE;
		
    setHasConfigurationInterface(true);
		setMinimumSize(QSizeF(32.0,48.0));
		    
		//this timer helps keeping devices informations up-to-date
		timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(refresh_devices_state()));   
		
		view=NULL;
}



void Mountoid::init()
{
		views.clear();
		
		views << new IconViewWidget(this,&devices,&settings);
		views << new ListViewWidget(this,&devices,&settings);
		views.at(0)->setVisible(FALSE);
		views.at(1)->setVisible(FALSE);
		
		set_view_mode(IconView);
	
		//parse fstab file
    ErrorState err=find_potential_devices(NULL,NULL);		
		
		if (err!=NoErr)
		{
         setFailedToLaunch(true, i18n(ErrorMessages[err]));
    }
		
		err=refresh_devices_state();
		
		if (err!=NoErr)
		{
         setFailedToLaunch(true, i18n(ErrorMessages[err]));
    }
		
		//self-explanatory
		read_settings();
		settings.initialized=TRUE;
 		apply_settings();
		
		//setup the original plasmoid shape and content		
		load_devices_into_views();
		resize(settings.size);		
		some_data_changed();		
}


//////////////////////////////////////////////////////////
/////////////////////////  devices management
//////////////////////////////////////////////////////////
void Mountoid::add_device (const QString &name,const QString &mount_pt,const QString &type)
{
	// we don't want to have the same device twice
	int n=devices.count();
	while ((n--) && (devices.at(n)->get_mount_point()!=mount_pt));
	if (n>=0)
		return;
		
	Device *dev = new Device(name,mount_pt,&settings);
	if (!dev)
		return;
	OperatingSystem OS;
	DeviceType t;
	bool remote;
	
	// try to guess automatically the OS type and FS type of the device
	dev->guess_type(type,t,OS,remote);
	
	dev->set_type(t);	
	dev->set_icon_name(DEVICE_TYPE_ICON[t]);
	dev->set_OS(OS);
	dev->set_OS_icon_name(OS_ICON[OS]);
	dev->set_load_command(DEVICE_LOAD_ACTION[t]);
	dev->set_eject_command(DEVICE_EJECT_ACTION[t]);
	dev->set_remote(remote);
	settings.order <<  devices.count();
	
	devices << dev;
	manage_device(dev);
	
}

void Mountoid::load_devices_into_views()
{
	if (devices.indexOf(NULL)>=0)
	{
		KMessageBox::error(NULL, QString("There's a NULL device..."), "BUG in mountoid.cpp" );	
		qDebug() << "There's a NULL device...";
		return;
	}
	for (int n=0; n<views.count(); n++)
	{
		views.at(n)->load_devices();
	}
}

void Mountoid::manage_device(Device *dev)
{	
	dev->set_managed(TRUE);
	load_devices_into_views();
	some_data_changed();
}

void	Mountoid::stop_managing_device			(Device *dev)
{	
	dev->stop_managing();
	load_devices_into_views();
	some_data_changed();	
}


void Mountoid::refresh_fstab()
{
	find_potential_devices(NULL,NULL);
}
		
ErrorState Mountoid::find_potential_devices(Device* dev, QString* l)
{
		// we wish to parse the fstab file
		QFile file("/etc/fstab");
		if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
		{
					return Cant_open_fstab;
		}

		QTextStream in(&file);
		while (!in.atEnd())
		{
			QString line = in.readLine();
			// we fetch the needed information from fstab lines : device name, mount point and fs type
			QRegExp regex("^([^\\s\\t]*)[\\s\\t]*([^\\s\\t]*)[\\s\\t]*([^\\s\\t]*).*$");   
			regex.indexIn(line);
			
			QString name = regex.cap(1);
			QString mount_pt = regex.cap(2);
			QString type = regex.cap(3);
			
			// only add "actual" devices, not system ones
			if ((name.indexOf("#")!=0) && !(name.isEmpty()) && (name!="none") && (mount_pt!="none") && (name!="proc"))
			{
				if (dev)
				{
					if (name == dev->get_dev_name())
					{
						*l=line;
						return NoErr;
					}
				}
				else					
					add_device(name,mount_pt,type);
			}
					 
		}
		some_data_changed();
		return NoErr;
}

void Mountoid::refresh_mtab()
{
		refresh_devices_state();
}

ErrorState Mountoid::refresh_devices_state()
{
		// we want to parse mtab file
		QFile file("/etc/mtab");
		if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
		{
					return Cant_open_mtab;
		}
		
		// first declare all device as unmounted, refreshing actual state further down, and also determine connection state
		int n=devices.count();
		while (n--)
		{
			devices.at(n)->set_state(Unmounted);
			devices.at(n)->update_connection_state();
		}

		QTextStream in(&file);
		while (!in.atEnd())
		{
			QString line = in.readLine();
			// we extract the relevant information from mtab lines
			QRegExp regex("^([^\\s\\t]*)[\\s\\t]*([^\\s\\t]*).*$");   
			regex.indexIn(line);
			
			QString name = regex.cap(1);
			QString mount_pt = regex.cap(2);
			
			// only take "actual" devices into account
			if ((name.indexOf("#")!=0) && !(name.isEmpty()) && (name!="none") && (mount_pt!="none") && (name!="proc"))
			{			
				// find the device with given mount point
				n=devices.count();
				while ((n--) && (mount_pt!=devices.at(n)->get_mount_point()));
				
				// then refresh the mount and connection states and the free space information accordingly
				if ((n>=0))
				{
					get_space_info(devices.at(n));
					devices.at(n)->set_state(Mounted);					
				}
			}			 
		}
		some_data_changed();
		return NoErr;	
}

ErrorState Mountoid::get_space_info(Device *dev)
{
	 // we fetch the information in statvfs
	 struct statvfs data;
   if((statvfs(dev->get_mount_point().toAscii(),&data)) < 0 ) 
	 {
		 return Cant_get_space_info;
   } 
	 else 
	 {
		 dev->set_free_space(((quint64)data.f_bfree)*data.f_bsize);
		 dev->set_max_space(((quint64)data.f_blocks)*data.f_bsize);
   }
	 return NoErr;
}


void Mountoid::show_entry(Device* dev)
{
	QString entry;
	if (find_potential_devices(dev,&entry) == NoErr)
		KMessageBox::information(NULL, QString("This is the fstab entry for the device at %1 <br><center><B>%2</B></center><br>").arg(dev->get_mount_point()).arg(entry) , QString("fstab entry") );
}


/////////////////////////////////////////////////////
//////////		data handling and syncing
////////////////////////////////////////////////////

void Mountoid::some_text_changed()
{
		// text changed so we need to tell the devices to adapt their (font) size
		refresh_content();		
}

void	Mountoid::some_data_changed					()
{
		if (!settings.initialized)
			return;
		// refresh devices display according to new data or settings
		int n= devices.count();
		while (n--) 
			devices.at(n)->some_data_changed();			
		if (view)
			view->some_data_changed();
		refresh_content();			
}

/////////////////////////////////////////////////////
/////////   appearence
/////////////////////////////////////////////////////
void Mountoid::refresh_content()
{
	resize(size());
	if (view)
		view->refresh_size();
	update();
}


void Mountoid::resizeEvent(QGraphicsSceneResizeEvent* event)
{
	if (view)
		view->resizeEvent(event);
	Plasma::Applet::resizeEvent(event);
}

void Mountoid::set_display_mode(const DisplayMode &M)
{
	if (view)
		view->set_display_mode(M);
	settings.appearance.display_mode=M;
	refresh_content();
}

void Mountoid::set_view_mode(const ViewMode &M)
{
	if (M>views.count())
		return;
	
	settings.appearance.view_mode=M;
	
	if (view)
	{
		disconnect (view,SIGNAL(SIG_refresh_mtab()),this,SLOT(refresh_mtab()));
		disconnect (view,SIGNAL(SIG_refresh_fstab()),this,SLOT(refresh_fstab()));
		disconnect (view,SIGNAL(SIG_settings_dialog()),this,SLOT(showConfigurationInterface()));
		disconnect (view,SIGNAL(SIG_stop_managing(Device *)),this,SLOT(stop_managing_device(Device*)));
		disconnect (view,SIGNAL(SIG_manage_device(Device*)),this,SLOT(manage_device(Device*)));
		disconnect (view,SIGNAL(SIG_set_view_mode(ViewMode)),this,SLOT(set_view_mode(ViewMode)));
		disconnect (view,SIGNAL(SIG_set_display_mode(DisplayMode)),this,SLOT(set_display_mode(DisplayMode)));
		disconnect (view,SIGNAL(SIG_show_entry(Device*)),this,SLOT(show_entry(Device*)));
		view->setVisible(FALSE);	
	}
	view=views[M];	
 	view->setVisible(TRUE);	
	setMinimumSize(view->get_minimum_size());
	connect (view,SIGNAL(SIG_refresh_mtab()),this,SLOT(refresh_mtab()));
	connect (view,SIGNAL(SIG_refresh_fstab()),this,SLOT(refresh_fstab()));
	connect (view,SIGNAL(SIG_settings_dialog()),this,SLOT(showConfigurationInterface()));
	connect (view,SIGNAL(SIG_stop_managing(Device *)),this,SLOT(stop_managing_device(Device*)));
	connect (view,SIGNAL(SIG_set_view_mode(ViewMode)),this,SLOT(set_view_mode(ViewMode)));
	connect (view,SIGNAL(SIG_manage_device(Device*)),this,SLOT(manage_device(Device*)));
	connect (view,SIGNAL(SIG_set_display_mode(DisplayMode)),this,SLOT(set_display_mode(DisplayMode)));
	connect (view,SIGNAL(SIG_show_entry(Device*)),this,SLOT(show_entry(Device*)));
	
	view->setPos(QPointF());
	view->load_devices();
	refresh_content();
}

//////////////////////////////////////////////////////
////////   configuration and settings management
/////////////////////////////////////////////////////

//////////////////// Uncommented plain self-explanatory boring stuff

ErrorState Mountoid::write_settings() const
{ 	
	KConfigGroup general = config("General");
		general.writeEntry("version",MOUNTOID_VERSION);
		general.writeEntry("size",size());
		general.writeEntry("refresh",settings.preferences.refresh_rate);
		general.writeEntry("use_best_unit",settings.preferences.use_best_unit);
		general.writeEntry("size_unit",(int)settings.preferences.size_unit);
		general.writeEntry("precision",settings.appearance.digits);
		general.writeEntry("show_free_space",settings.preferences.is_free_space);
		general.writeEntry("show_used_space",settings.preferences.is_used_space);
		general.writeEntry("show_max_space",settings.preferences.is_max_space);
		general.writeEntry("view_mode",(int)settings.appearance.view_mode);
			
	KConfigGroup pref = config("Preferences");
		pref.writeEntry("draw_background",settings.appearance.show_background);
		pref.writeEntry("display_mode",(int)settings.appearance.display_mode);
		pref.writeEntry("OS_icon_size",settings.appearance.OS_size);
		pref.writeEntry("mounting_icon_size",settings.appearance.mounting_size);
		pref.writeEntry("connect_icon_size",settings.appearance.connect_size);
		pref.writeEntry("font",settings.appearance.font);
		pref.writeEntry("font_space",settings.appearance.font_space);
		pref.writeEntry("devices_order",settings.order);
		pref.writeEntry("ping_timeout",(int)settings.preferences.mount_timeout);
		pref.writeEntry("mount_timeout",(int)settings.preferences.ping_timeout);
		int n=N_COLORS;
		while (n--)
			pref.writeEntry(QString("color_%1").arg(n),settings.appearance.colors.at(n));
	
	
		n = devices.count();
		while (n--)
		{	
			Device *dev = devices.at(n);
			KConfigGroup dev_grp = config(dev->get_dev_name());
				dev_grp.writeEntry("nick_name",dev->get_nick_name());
				dev_grp.writeEntry("icon",dev->get_icon_name());
				dev_grp.writeEntry("type",(int)dev->get_type());
				dev_grp.writeEntry("managed",dev->is_managed());
				dev_grp.writeEntry("show_space",dev->is_show_free_space());
				dev_grp.writeEntry("OS",(int)dev->get_OS());
				dev_grp.writeEntry("OS_icon",dev->get_OS_icon_name());
				dev_grp.writeEntry("sudo_mount",dev->is_sudo());	
				dev_grp.writeEntry("load_command",dev->get_load_command());
				dev_grp.writeEntry("eject_command",dev->get_eject_command());
				dev_grp.writeEntry("ip_address",dev->get_ip_address());
				dev_grp.writeEntry("remote",dev->is_remote());
				dev_grp.writeEntry("show_connect_icon",dev->is_show_icon_state());
		}
		
	return NoErr;
}

ErrorState Mountoid::read_settings()
{
	
	KConfigGroup general = config("General");
		bool reinit_colors = FALSE; // (general.readEntry("version",MOUNTOID_VERSION)!=MOUNTOID_VERSION);
		settings.size=general.readEntry("size",QSizeF(100.0,100.0));
		settings.preferences.refresh_rate=general.readEntry("refresh",30);
		settings.preferences.use_best_unit=general.readEntry("use_best_unit",TRUE);
		settings.preferences.size_unit=(Unit)general.readEntry("size_unit",(int)KB);
		settings.appearance.digits=general.readEntry("precision",1);
		settings.preferences.is_free_space=general.readEntry("show_free_space",TRUE);
		settings.preferences.is_used_space=general.readEntry("show_used_space",FALSE);
		settings.preferences.is_max_space=general.readEntry("show_max_space",TRUE);
		settings.appearance.view_mode=(ViewMode)general.readEntry("view_mode",(int)IconView);
		
	KConfigGroup pref = config("Preferences");
		settings.appearance.show_background=pref.readEntry("draw_background",TRUE);
		settings.appearance.display_mode=(DisplayMode)pref.readEntry("display_mode",(int)StackedHorizontally);
		settings.appearance.OS_size=pref.readEntry("OS_icon_size",16);
		settings.appearance.mounting_size=pref.readEntry("mount_icon_size",32);
		settings.appearance.connect_size=pref.readEntry("connect_icon_size",16);
		settings.appearance.font=pref.readEntry("font",KGlobalSettings::generalFont());
		settings.appearance.font_space=pref.readEntry("font_space",KGlobalSettings::generalFont());
		settings.preferences.mount_timeout=general.readEntry("mount_timeout",60);
		settings.preferences.ping_timeout=general.readEntry("ping_timeout",50);
		QList<int> order= pref.readEntry("devices_order",settings.order);
		if (order.count() == settings.order.count())
			settings.order=order;
		int n=0;
		while (n<N_COLORS)
			if (reinit_colors)
				settings.appearance.colors << QColor(COLORS_DEFAULT[n++]);
			else
			{
				settings.appearance.colors << pref.readEntry(QString("color_%1").arg(n),QColor(COLORS_DEFAULT[n]));
				n++;
			}
			
		n = devices.count();
		while (n--)
		{	
			Device *dev=devices.at(n);
			KConfigGroup dev_grp = config(dev->get_dev_name());
				dev->set_nick_name(dev_grp.readEntry("nick_name",dev->get_nick_name()));
 				dev->set_icon_name(dev_grp.readEntry("icon",dev->get_icon_name()));
 				dev->set_type((DeviceType)dev_grp.readEntry("type",(int)dev->get_type()));
				dev->set_managed(dev_grp.readEntry("managed",TRUE));
				dev->set_show_free_space(dev_grp.readEntry("show_space",TRUE));
				dev->set_OS((OperatingSystem)dev_grp.readEntry("OS",(int)dev->get_OS()));
				dev->set_OS_icon_name(dev_grp.readEntry("OS_icon",dev->get_OS_icon_name()));
				dev->set_sudo(dev_grp.readEntry("sudo_mount",FALSE));
				dev->set_load_command(dev_grp.readEntry("load_command",dev->get_load_command()));
				dev->set_eject_command(dev_grp.readEntry("eject_command",dev->get_eject_command()));
				dev->set_ip_address(dev_grp.readEntry("ip_address",""));
				dev->set_remote(dev_grp.readEntry("remote",dev->is_remote()));
				dev->set_show_icon_state(dev_grp.readEntry("show_connect_icon",dev->is_show_icon_state()));
		}
		
 	return NoErr;	
}




void Mountoid::createConfigurationInterface(KConfigDialog *parent) 
{
		connect(parent, SIGNAL(applyClicked()),   this, SLOT(apply_config()));
		connect(parent, SIGNAL(okClicked()),      this, SLOT(apply_config()));

		parent->showButton(KConfigDialog::Help, FALSE);
		parent->showButton(KConfigDialog::Apply, TRUE);

		config_devices = new Dia_Devices (NULL);
		config_devices->set_value(devices,settings.order);
		parent->addPage(config_devices, i18n("Devices"), "drive-harddisk");
		
		config_appearance = new Dia_Appearance (NULL);
		config_appearance->set_value(settings.appearance);
		parent->addPage(config_appearance, i18n("Appearance"), "preferences-desktop");		
		
		config_preferences = new Dia_Preferences (NULL);
		config_preferences->set_value(settings.preferences);
		parent->addPage(config_preferences, i18n("Preferences"), "preferences-system");		
}

void Mountoid::apply_config()
{	
	settings.appearance=config_appearance->get_value();
	settings.preferences=config_preferences->get_value();
	
 	QList<DeviceInfo> list=config_devices->get_value();
	settings.order=config_devices->get_order();	
	
	int n=list.count();
	
	while (n--)
	{
		
		if (devices.at(n)->is_managed() && (!list.at(n).managed))
		{
			devices.at(n)->set_info(list.at(n));
			stop_managing_device(devices.at(n));
		}
		else if (!devices.at(n)->is_managed() && (list.at(n).managed))
		{
			devices.at(n)->set_info(list.at(n));
			manage_device(devices.at(n));
		}
		else
			devices.at(n)->set_info(list.at(n));		
	}
	
	apply_settings();
	refresh_devices_state();
	some_data_changed();
}

void Mountoid::apply_settings()
{
	if (settings.appearance.show_background)
			setBackgroundHints(DefaultBackground);
		else
			setBackgroundHints(NoBackground);
	timer->start(1000*settings.preferences.refresh_rate);
	
	set_view_mode(settings.appearance.view_mode);
	write_settings();
}


Mountoid::~Mountoid()
{
 	delete timer;
	view->stop_animations();
	
		if (hasFailedToLaunch())
    {
      
    } 
		else
    {
        write_settings();
    }
		
 		while (devices.count())
 			delete devices.takeFirst();	
}


#include "mountoid.moc"
