/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#ifndef MOUNTOID_HEADER
#define MOUNTOID_HEADER
////////////////////////
#include <Plasma/Applet>
////////////////////////
#include "settings.h"
////////////////////////
class QTimer;
class Device;
class KConfigDialog;
class MountoidView;
class Dia_Devices;
class Dia_Appearance;
class Dia_Preferences;

//////////////////////
//  General defines	//
//////////////////////
#define 			MOUNTOID_VERSION						"0.40"

///////////////////////////////////////////////////////
// This class is the applet main class

class Mountoid : public Plasma::Applet
{
    Q_OBJECT
    public:
																Mountoid											(QObject *parent, 
																																const QVariantList &args);
																~Mountoid											(); 
				//  does initialization stuff
        void 										init													();
				
				
	protected:
				// manages custom pages for the settings dialog
				void   									createConfigurationInterface	(KConfigDialog *parent);
				// handles resize events (devices need to set their width accordingly)
				virtual void						resizeEvent 									( QGraphicsSceneResizeEvent * event );
				// when some content changes, sync devices data and view's size
				void										refresh_content								();
				
	private:
				// sync applet properties when the settings are modified
				void 										apply_settings								();
				// saves applet settings to KConfig
				ErrorState							write_settings								()	const;
				// fetches applet settings from KConfig
				ErrorState							read_settings									();
				// creates/refreshes the devices entries in the view
				void 										load_devices_into_views				();
				// creates the device with given name, mount point and fs typedef
				// and appends it to the devices list if it is not already there
				void										add_device 										(const QString &name,
																															const QString &mount_pt,
																															const QString &type);
				// to schedule regular devices informations refresh
				QTimer* 								timer;
				// contains the list of available devices
				QList<Device*>					devices;	
				// pages to add to the settings dialog
				Dia_Devices							*config_devices;
				Dia_Preferences					*config_preferences;
				Dia_Appearance					*config_appearance;
				// the configuration informations
				SettingsStructure				settings;
				// Widgets to display devices				
				QList<MountoidView*>		views;
				// current widget
				MountoidView						*view;
				
				
		private slots:
				// checks devices mount state by parsing the mtab file
				// and also calls get_space_info
				ErrorState	 						refresh_devices_state					();
				// parses fstab file to get available physical devices (not system ones)
				// if dev is not NULL then extract the corresponding line from fstab and store it into l
				// (in that case no other action is performed)
				// otherwise fill the device list with detected devices
				ErrorState							find_potential_devices				(Device* dev, QString *l);
				// fetches the given device maximum and free space
				ErrorState							get_space_info								(Device *);
				// displays in a dialog box the fstab entry for the device
				void										show_entry										(Device *);
				// called when some text changed and new font sizes need to be computed anew
				void										some_text_changed							();
				// called when the user requests a reparsing of fstab
				void										refresh_fstab									();
				// called when the user requests a reparsing of mtab
				void										refresh_mtab									();				
				// called to order the devices to refresh their data and their GraphicsItems
				void										some_data_changed							();
				// called when the user requests to stop managing the given device
				void										stop_managing_device					(Device *);
				// called when the user requests to manage/display the given device
				void 										manage_device									(Device *);
				// called by the settings dialog when it is accepted
				// stores the new settings into the current structure
				// calls some_data_changed() and apply_settings()
				void										apply_config									();
				// swaps between list and icon view
				void										set_view_mode									(const ViewMode &);
				// select a display mode
				void										set_display_mode							(const DisplayMode &);
};
 
// This is the command that links the applet to the .desktop file
K_EXPORT_PLASMA_APPLET(mountoid, Mountoid)
#endif
