/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#include <QGraphicsSceneResizeEvent>
#include <QGraphicsSceneMouseEvent>
#include <QMenu>
#include <KMessageBox>
#include <QDebug>
#include <QAction>
#include <KIcon>
// #include <QDebug>
///////////////////////////////
#include "mountoid_view.h"
#include "dia_change_nick_name.h"
////////////////////////////////

/////////////////////////////////////////////////////////////
/////         																					/////
/////     Class MountoidView														/////
/////         																					/////
/////////////////////////////////////////////////////////////

MountoidView::MountoidView(QGraphicsWidget *par, QList< Device* >* list, SettingsStructure* set,const ViewMode &vm): QGraphicsWidget(par)
{
	mode=vm;
	devices=list;
	settings=set;
	widget=NULL;
	is_resizing=FALSE;
	
	connect (&animation , SIGNAL(timeout()) , this, SLOT(animation_next_step()));
	animation.setInterval(ANIMATION_INTERVAL);	
}


////////////////////////////
// data management
///////////////////////////

void MountoidView::connect_device(Device *dev)
{
	disconnect(dev,SIGNAL(SIG_refresh_mtab()),this,SIGNAL(SIG_refresh_mtab()));
	connect(dev,SIGNAL(SIG_refresh_mtab()),this,SIGNAL(SIG_refresh_mtab()));
				
	disconnect(dev,SIGNAL(SIG_animation_changed()),this,SLOT(check_animation_needed()));
	connect(dev,SIGNAL(SIG_animation_changed()),this,SLOT(check_animation_needed()));
}


void MountoidView::change_nick_name(Device* dev)
{
	Dia_ChangeNickName diag(NULL);
	diag.set_value(dev->get_nick_name());
	if (diag.exec() == QDialog::Accepted)
	{
		dev->set_nick_name(diag.get_value());
		some_data_changed();
	}
}


void MountoidView::icon_clicked(Device *dev)
{
		if (dev->get_state() == Mounted)
		{
			dev->launch_file_manager();
		}
		else if ((dev->get_connection_state() == Ready) || (dev->get_connection_state() == Unavailable))
		{
			dev->do_mount();
		}
		else if (dev->get_connection_state() != Ready)
		{
			dev->do_load();
		}
}



int MountoidView::number_managed						() const
{
	int  n=0;
	int m=devices->count();
	while (m--)
		if (devices->at(m)->is_managed())
			n++;
		
	return n;
}


SettingsStructure* MountoidView::get_settings() const
{
	return settings;
}


QList< Device* >* MountoidView::get_devices() const
{
	return devices;
}


Device* MountoidView::mapped_device(const int& n)
{
	if ((n<0) || (n>=settings->order.count()) || (settings->order.at(n)<0) || (settings->order.at(n)>=devices->count()))
	{
		KMessageBox::error(NULL, QString("Out of range error..."), "BUG in mountoid_view.cpp" );	
		qDebug() << "n=" << n << " -- order=" << settings->order;
		return NULL;
	}
	return devices->at(settings->order.at(n));
}


void MountoidView::some_data_changed()
{
	
	for (int n=0; n<devices->count(); n++)
		some_data_changed(n);
	refresh_size();
	
}


int MountoidView::get_index(Device* dev)
{
	int n=devices->count();
	while ((n--) && (devices->at(n)!=dev));
	
	return n;
}


/////////////////////////////////
// appearence
//////////////////////////////////
void MountoidView::stop_animations()
{
	animation.stop();
}

void MountoidView::check_animation_needed()
{
	int n=devices->count();
	bool ok = true;
	while ((n--) && ok)
		ok = !(devices->at(n)->is_animated());
	if (ok)
		stop_animations();
	else
		animation.start();		
}


QRectF MountoidView::boundingRect() const
{
    return parentItem()->boundingRect();
}


void MountoidView::refresh_size()
{
	resize(boundingRect().size());
	QSizeF S= size();
	apply_new_size(S);	
	((QGraphicsWidget*)parentItem())->resize(S);
	update();
}

void	MountoidView::resizeEvent ( QGraphicsSceneResizeEvent * event )
{	
	if (!settings->initialized)
			return;
	
	// when changing size one needs to compute the new height of displayed devices
	// and resizing accordingly afterwards. Need thus to take care about infinite loop	
	if (!is_resizing)
	{	
		is_resizing=TRUE;
		QSizeF siz=event->newSize();
		apply_new_size(siz);
		// do the standard stuff
		QGraphicsWidget::resizeEvent(event);
		// and resize the parent widget to the optimal size
		// which calls back this proc 
		((QGraphicsWidget*)parentItem())->resize(siz);
	}
	else
	{
		// do the standard stuff
		QGraphicsWidget::resizeEvent(event);		
		is_resizing=FALSE;
	}
}

////////////////////////
//  context menu
////////////////////////
void MountoidView::context_menu	(QGraphicsSceneContextMenuEvent *ev , Device *par_dev)
{
		QMenu menu;      
		
		QAction *A_mount=NULL;
		QAction *A_load=NULL;
		QAction *A_file_m=NULL;
		QAction *A_nick=NULL;
		QAction *A_managed=NULL;
		QList<QAction *> A_dev;
		
		if (par_dev)
		{
			if (par_dev->get_state() == Unmounted)
			{
				A_mount= menu.addAction(KIcon("emblem-mounted"), "Mount");
				A_mount->setDisabled( (par_dev->check_connection_state()!=Ready) && (par_dev->is_remote()));
			}
			else if (par_dev->get_state() == Mounted)
				A_mount= menu.addAction(KIcon("emblem-unmounted"), "Unmount");
			else
				A_mount= menu.addAction(KIcon("process-stop"), "Stop un/mount operation");
			
			if (par_dev->is_remote())
			{			
				if (par_dev->check_connection_state()==Off)
					A_load=menu.addAction(KIcon("network-connect") , "Turn on");
				else if ((par_dev->check_connection_state()==On) || (par_dev->check_connection_state()==Ready))
					A_load=menu.addAction(KIcon("network-disconnect") , "Turn off");
				else if ((par_dev->check_connection_state()==Connecting) || (par_dev->check_connection_state()==Disconnecting))
					A_load= menu.addAction(KIcon("process-stop"), "Stop dis/connection operation");			
			}
			else if ((par_dev->get_type()==CDRom) && (par_dev->check_connection_state()==Off))
				A_load=menu.addAction(KIcon("media-playback-start") , "Load tray");
			else if ((par_dev->get_type()==CDRom) && (par_dev->check_connection_state()==On))
				A_load=menu.addAction(KIcon("media-eject") , "Eject tray");
			
			menu.addSeparator(); 
			A_file_m=menu.addAction(KIcon("konqueror"),"Open in a file manager");
			A_file_m->setEnabled(par_dev->get_state() == Mounted);
			menu.addSeparator();
			A_nick = menu.addAction(KIcon("edit-rename"), "Change device nickname");
			A_managed = menu.addAction(KIcon("dialog-cancel"), "Stop managing this device");
			QMenu *M0 = menu.addMenu(KIcon("device-notifier"),"Manage other devices");
			
			for (int n=0; n < devices->count();n++)
				if (!devices ->at(n)->is_managed())
					A_dev << M0->addAction(KIcon(devices->at(n)->get_icon_name()),devices->at(n)->get_mount_point());
			menu.addSeparator();
		}
		QAction *A_entry = menu.addAction(KIcon(""),"Show fstab entry for the device");
		QAction	*A_refresh_m = menu.addAction(KIcon("view-refresh"),"Reload devices mount state");
		QAction	*A_refresh_f = menu.addAction(KIcon("view-refresh"),"Reload devices list from fstab");
		menu.addSeparator();
		QMenu		*M1 = menu.addMenu(KIcon("video-display"),"Set display mode");
		QList<QAction *> A_display;
		for (int n=0; n<N_DISPLAY_MODE;n++)
			if (DISPLAY_MODE_VIEW[n]==mode)
			{
				QAction *tmp=M1->addAction(KIcon(DISPLAY_MODE_ICON[n]),DISPLAY_MODE_NAME[n]);
				tmp->setEnabled(n != settings->appearance.display_mode);
				A_display << tmp;
			}
		M1->setEnabled(A_display.count());
		
		QMenu		*M = menu.addMenu(KIcon("view-choose"),"Set view mode");
		QAction *A_view[N_VIEW_MODE];
		for (int n=0; n<N_VIEW_MODE; n++)
		{
			A_view[n]=M->addAction(KIcon(VIEW_MODE_ICON[n]),VIEW_MODE_NAME[n]);
			A_view[n]->setCheckable(TRUE);
			A_view[n]->setChecked(n==mode);
			A_view[n]->setEnabled(n!=mode);
		}
		
		QAction *A_settings = menu.addAction(KIcon("configure"), "Open settings dialog");
		
		QAction *a = menu.exec(ev->screenPos());
		
		if (a==NULL)
			return;
		
		if (a == A_mount)
			par_dev->do_mount();
		else if (a == A_nick)
			change_nick_name(par_dev);
		else if (a == A_managed)
			emit SIG_stop_managing(par_dev);
		else if (a == A_entry)
			emit SIG_show_entry(par_dev);
		else if (a == A_refresh_m)
			emit SIG_refresh_mtab();
		else if (a == A_refresh_f)		
			emit SIG_refresh_fstab();
		else if (a == A_settings)
			emit SIG_settings_dialog();
		else if (a == A_file_m)
			par_dev->launch_file_manager();
		else if (a == A_load)
			par_dev->do_load();
		else
		{
			bool ok=FALSE;
			int n=N_VIEW_MODE;
			while ((n--) &&!ok)
				if (a==A_view[n])
				{
					ok=TRUE;
					emit SIG_set_view_mode((ViewMode)n);
				}
			
			int k=0;
			n=0; 
			while (!ok && (n < devices->count()))
			{
				if (!devices ->at(n)->is_managed())
				{
					if (a==A_dev.at(k))
					{
						emit SIG_manage_device(devices->at(n));
						ok=TRUE;
					}
					k++;
				}
				n++;
			}
			
			k=0;
			n=0; 
			while (!ok && (n < N_DISPLAY_MODE))
			{
				if (DISPLAY_MODE_VIEW[n]==mode)
				{
					if (a==A_display.at(k))
					{
						emit SIG_set_display_mode((DisplayMode)k);
						ok=TRUE;
					}
					k++;
				}
				n++;
			}
		}
			
		
}
