/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#ifndef MOUNTOID_VIEW_HEADER
#define MOUNTOID_VIEW_HEADER
/////////////////////////////
#include <QGraphicsWidget>
#include <QTimer>
/////////////////////////////
#include "device.h"
/////////////////////////////

// To take care of the border thickness of the standard applet background
#define 			PLASMA_ADJUST								16.0
// GraphicsItems padding
#define 			ICON_SPACING								4.0

// interval in msec for mount/load animations
#define	ANIMATION_INTERVAL								100

////////////////////////////////////////////////////////////////////////////////
// this class is the abstract class of widgets containing and displaying devices
// it has methods/signals to interact with the applet
// and provides a contextual menu
// all interactions from the user are done through this widget


class MountoidView : public QGraphicsWidget
{
	Q_OBJECT
	public:
													MountoidView									(QGraphicsWidget *par, 
																												 QList<Device*> *list ,
																												 SettingsStructure *set,
																												 const ViewMode &VM
																												 );
		
		// ensures that the devices get the same amount of space
		// and that the applet's size is optimum
		// postions the devices according ot their display rank
		void									refresh_size									();
		virtual		void 				apply_new_size								(QSizeF &)=0;
		// handles resize events (devices need to set their width accordingly)
		virtual void					resizeEvent 									( QGraphicsSceneResizeEvent * event );
		// called to populate the icon view
		virtual	void					load_devices									()=0;
		// connect slots and signals between managed devices and this widget
		void									connect_device								(Device *);
		// something changed for the device of given rank, keep track of it
		virtual void					some_data_changed							(const int &)=0;
		// change between different display types
		virtual void					set_display_mode							(const DisplayMode &)=0;
		// minimum size of the view
		virtual QSizeF				get_minimum_size							()=0;
		// something changed, call some_data_changed(device) for all devices
		void									some_data_changed							();
		// the extent of the view
		virtual	QRectF				boundingRect									() const;
		// returns the pointer on the list of devices
		QList<Device *>				*get_devices									() const;
		// returns the pointer on the settings of the applet
		SettingsStructure			*get_settings									() const;
		// this function returns the device of given display rank
		Device								*mapped_device								(const int&);
		// returns the index of a given device
		int										get_index											(Device *);
		

		
	protected:
		// when another widget do the display (e.g. a listview)
		QGraphicsWidget				*widget;
		// the devices list pointing on the list of the applet
		QList<Device*>				*devices;
		// the pointer on the applet's settings
		SettingsStructure 		*settings;
		// the view mode corresponding to the actual MountoidView
		ViewMode							mode;
		// to avoid infinite loop when computing new optimal size
		bool									is_resizing;		
		// how many devices are displayed
		int										number_managed								() const;		
		// timer to manage animations
		QTimer								animation;
				
				
	public slots:
		// pops up a dialog to edit the nickname of the given device
		void									change_nick_name						(Device*);
		// builds the popup contextual menu
		void									context_menu								(QGraphicsSceneContextMenuEvent*,Device*);
		// when the users clicks on a device, executes the required action with respect to its mount state
		// mounts it when unmounted
		// launches a file manager when mounted
		void									icon_clicked								(Device *);
		// slot receiving animation calls
		virtual void					animation_next_step					()=0;
		// stops the timer when not needed
		void									check_animation_needed			();
    void 									stop_animations							();
				
	signals:
		// emited when the user requests a refresh of mount state 
		void 									SIG_refresh_mtab						();
		// emited when the user requests a refresh of available devices
		void 									SIG_refresh_fstab						();
		// emited when the user requests the settings dialog
		void 									SIG_settings_dialog					();
		// emited when the user requests to stop managing this device
		void									SIG_stop_managing						(Device *);
		// emited when the user requests to start managing this device
		void									SIG_manage_device						(Device *);
		// emited when the user requests to view the fstab entry for the device
		void									SIG_show_entry							(Device *);
		//	emited when the user changes the view mode
		void									SIG_set_view_mode						(ViewMode);
		//	emited when the user changes the display mode
		void									SIG_set_display_mode				(DisplayMode);
	
};

#endif
