/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#ifndef SETTINGS_HEADER
#define	SETTINGS_HEADER
////////////////////////////
#include <QList>
#include <QString>
#include <QFont>
#include <QColor>
#include <QSizeF>
///////////////////////////

///////////////////////
//   Error events	 ///
/////////////////////
typedef enum ErrorState
{
		NoErr,
		Cant_open_mtab,
		Cant_open_fstab,
		Cant_open_device,
		Cant_get_space_info
};

static const char* ErrorMessages[] =
{
		"no error",
		"can't open mtab file for reading",
		"can't open fstab file for reading",
		"can't open file system %1, check rights",
		"can't get free space information"
};

/////////////////////////////
//			 view modes	  		//
///////////////////////////
#define N_VIEW_MODE 2

typedef enum  ViewMode
{
	IconView,
	ListView
};

static const char* VIEW_MODE_NAME []=
{
	"icon view",
	"list view"
};

static const char* VIEW_MODE_ICON []=
{
	"view-list-icons",
	"view-list-details"
};


///////////////////////////
//	 Display mode				//
/////////////////////////
#define N_DISPLAY_MODE 2


typedef enum DisplayMode
{
	StackedVertically,
	StackedHorizontally
};

static const char* DISPLAY_MODE_NAME[]=
{
	"Stack icons vertically",
	"Stack icons horizontally"
};

static const char* DISPLAY_MODE_ICON[]=
{
	"distribute-vertical-margin",
	"distribute-horizontal-margin"
};

static const ViewMode DISPLAY_MODE_VIEW[]=
{
	IconView,
	IconView
};

///////////////////////////////////
// Units to express disk sizes	//
/////////////////////////////////
#define N_UNITS 5

typedef enum Unit
{
	B,KB,MB,GB,TB
};

static const char* UNIT_NAME[]=
{
		"B","KB","MB","GB","TB"
};

#define FREE_TEXT "free:"
#define USED_TEXT "used:"
#define MAX_TEXT 	"total:"

// Given a size this function expresses it in the most convenient unit
// (i.e. the unit in which the size is X,F where 0<X<1024 and F is the fractional part)
// definition in file mountoid.cpp
extern void to_best_unit (Unit &unit , float &size);

// Given a size this function expresses it in the given unit
// definition in file mountoid.cpp
extern void to_unit (Unit unit , float &size);



//////////////////////////////////////
//  Application settings structure	//
//////////////////////////////////////
struct AppearanceStructure
{
	public:
		// the font to display devices nickname and free/used/max space
		QFont						font;
		QFont						font_space;
		// the colors used in the application
		QList<QColor>		colors;
		// does the applet draw its background ?
		bool						show_background;
		// display mode
		DisplayMode			display_mode;
		// how many digits to display floats in sizes
		int							digits;	
		// the size of the OS icon in pixels
		int							OS_size;
		// the size of the connect icon in pixels
		int							connect_size;
		// the size of the mounting icon in pixels
		int							mounting_size;
		// the kind of view
		ViewMode				view_mode;
		
};

struct PreferenceStructure
{
	public:
		// does tha device space information show free/used/max space
		bool								is_free_space;
		bool								is_used_space;
		bool								is_max_space;
		// rate in seconds to refresh devices informations
		int									refresh_rate;
		// does the applet express sizes in the best unit ? 
		bool								use_best_unit;
		// or else in which unit ?
		Unit								size_unit;
		// timeout in sec to achieve un/mount commands
		int									mount_timeout;
		// timeout in msec to achieve ping commands
		int									ping_timeout;
};

struct SettingsStructure
{
	public:
		// all about visual preferences
		AppearanceStructure	appearance;
		PreferenceStructure	preferences;
		// remember the size of the applet as KDE has problems dealing with that
		QSizeF							size;
		// display rank of the devices
		QList<int>					order;
		
		// global variables (crappy hack, sorry)
		float								text_height;		
		bool								initialized;
};


////////////////////////////////////////
//				  Application colors				//
////////////////////////////////////////
#define 	N_COLORS 	18

enum Color
{
	ToolTipText,
	ToolTipNickName,
	ToolTipMountPoint,
	ToolTipName,
	ToolTipStatus,
	ToolTipFreeSpace,
	ToolTipMaxSpace,
	NickName,
	EmptyDevice,
	HalfFullDevice,
	FullDevice,
	ToolTipUsedSpace,
	NickNameFreeSpace,
	NickNameUsedSpace,
	NickNameMaxSpace,
	ReadyDevice,
	OnDevice,
	OffDevice
};

static const Color COLOR_ORDER[]=
{
	
	NickName,
	NickNameFreeSpace,
	NickNameUsedSpace,
	NickNameMaxSpace,
	EmptyDevice,
	HalfFullDevice,
	FullDevice,
	ReadyDevice,
	OnDevice,
	OffDevice,
	ToolTipText,
	ToolTipNickName,
	ToolTipMountPoint,
	ToolTipName,
	ToolTipStatus,
	ToolTipFreeSpace,
	ToolTipUsedSpace,
	ToolTipMaxSpace
};

static const char* COLORS_NAME[]=
{
	"Tooltip text",
	"Tooltip nickname",
	"Tooltip mount point",
	"Tooltip device name",
	"Tooltip mount status",
	"Tooltip free space",
	"Tooltip device capacity",
	"Nickname",
	"Device empty",
	"Device half-full",
	"Device full",
	"Tooltip used space",
	"Nickname free space",
	"Nickname used space",
	"Nickname device capacity",
	"Network device ready",
	"Network device on",
	"Network device off",
};

static const char* COLORS_DEFAULT[]=
{
	"#000000",
	"#400050",
	"#8000A0",
	"#60007A",
	"#A000F0",
	"#108010",
	"#084008",
	"#FFFFFF",
	"#00FF00",
	"#A0C000",
	"#FF0000",
	"#801010",
	"#C0C0C0",
	"#B0B0B0",
	"#A0A0A0",
	"#00FF00",
	"#A0C000",
	"#FF0000"
};


#endif