/***************************************************************************
****************************************************************************
**               Copyright (C) 2009 by Warzin ,  FYM Int.                 **
**                                                                        **
**                        warzin.warzin@gmail.com                         **
**                                                                        **
****************************************************************************
****************************************************************************
********                                                          **********
********              KDE Plasma applet  Mountoid                 **********
********                                                          **********
****************************************************************************
****************************************************************************
**                                                                        **
**   This program is free software; you can redistribute it and/or modify **
**   it under the terms of the GNU General Public License as published by **
**   the Free Software Foundation; either version 2 of the License, or    **
**   (at your option) any later version.                                  **
**                                                                        **
**   This program is distributed in the hope that it will be useful,      **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of       **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
**   GNU General Public License for more details.                         **
**                                                                        **
**   You should have received a copy of the GNU General Public License    **
**   along with this program; if not, write to the                        **
**   Free Software Foundation, Inc.,                                      **
**   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            **
**                                                                        **
****************************************************************************
****************************************************************************/
#include <math.h>
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <KGlobalSettings>
//////////////////////////////////
#include "text_widget.h"
#include "settings.h"
#include "device.h"
#include "icon_view.h"
//////////////////////////////////


/////////////////////////////////////////////////////////////
/////         																					/////
/////     Class TextWidget															/////
/////         																					/////
/////////////////////////////////////////////////////////////

TextWidget::TextWidget(Device *par  , IconViewWidget *vpar) : QGraphicsItem()
{
	par_dev=par;		
	view=vpar;
	refresh_font();
	setZValue(1.0);
}

void TextWidget::set_width(const float &w)
{
	allocated_width=w;
	if (!isVisible() || par_dev->get_nick_name().isEmpty())
			return;
	
	// we want to determine the best font size to fit the text in a given width
	// starting from the application font...
	refresh_font();
	
	int pt1 = font.pointSize();	
	QFontMetricsF fm(font);
	
	// ... and going down to the largest fitting font size	
	while ((pt1 >= 4) && (fm.width(par_dev->get_nick_name())>w))
	{
		pt1--;
		font.setPointSize(pt1);
		fm=QFontMetricsF(font);
	}
	
	// we do the same for the displayed space information texts
	int pt2 = font_space.pointSize();	
	while ((pt2>=4) && (get_space_width()>w))
	{
		pt2--;
		font_space.setPointSize(pt2);
	}
	
	// now define properly the display rects and update the display
	manage_rects();
	update();
}

float	TextWidget::get_space_width() const
{
	float max_w=0.0;
	
	// if the device is mounted and the user wants the device space information to appear, find the widest text amongst displayed text
	if (par_dev->is_show_free_space() && par_dev->get_state()==Mounted)
	{
		float w2,w3,w4;	
		QString fs,us,ms;
		par_dev->get_free_space_text(fs,us,ms);
		
		QFontMetricsF fm_space(font_space);
		w2=fm_space.width(QString("%2 %1").arg(fs).arg(FREE_TEXT));
		w3=fm_space.width(QString("%2 %1").arg(us).arg(USED_TEXT));
		w4=fm_space.width(QString("%2 %1").arg(ms).arg(MAX_TEXT));
		if ((w2>max_w) && par_dev->settings->preferences.is_free_space)
			max_w=w2;
		if ((w3>max_w) && par_dev->settings->preferences.is_used_space)
			max_w=w3;
		if ((w4>max_w) && par_dev->settings->preferences.is_max_space)
			max_w=w4;
	}
		
	return max_w;
}

//////////////////// Uncommented plain self-explanatory boring stuff
float	TextWidget::get_height() const
{
	return rect_nick_name.height()+rect_free_space.height()+rect_used_space.height()+rect_max_space.height();
}

float	TextWidget::get_nick_name_height() const
{
	return rect_nick_name.height();
}

void TextWidget::refresh_font()
{
	font = par_dev->settings->appearance.font;
	font_space = par_dev->settings->appearance.font_space;
}

void TextWidget::manage_rects()
{
	QFontMetricsF fm(font);
	QFontMetricsF fm2(font_space);
	float cur_height = 0.0;
	rect_nick_name=QRectF(QPointF(0.0,cur_height),QSizeF(allocated_width,fm.height()));	
	cur_height+=fm.height();
	if (par_dev->settings->preferences.is_free_space && par_dev->is_show_free_space() && (par_dev->get_state()==Mounted))
	{
		rect_free_space=QRectF(QPointF(0.0,cur_height) , QSizeF(allocated_width, fm2.height()) );
		cur_height+=fm2.height();
	}
	else
		rect_free_space=QRectF();
	
	if (par_dev->settings->preferences.is_used_space && par_dev->is_show_free_space() && (par_dev->get_state()==Mounted))
	{
		rect_used_space=QRectF(QPointF(0.0,cur_height) , QSizeF(allocated_width, fm2.height()) );
		cur_height+=fm2.height();
	}
	else
		rect_used_space=QRectF();
	
	if (par_dev->settings->preferences.is_max_space && par_dev->is_show_free_space() && (par_dev->get_state()==Mounted))
	{
		rect_max_space=QRectF(QPointF(0.0,cur_height) , QSizeF(allocated_width, fm2.height()) );
		cur_height+=fm2.height();
	}
	else
		rect_max_space=QRectF();		
	
	
}

QRectF TextWidget::boundingRect() const
{		
	if (par_dev->settings->appearance.display_mode==StackedHorizontally)
		return QRectF(QPointF(),QSizeF(allocated_width,get_height()+par_dev->settings->text_height-get_nick_name_height()));
	else
		return QRectF(QPointF(),QSizeF(allocated_width,get_height()));
}



void TextWidget::paint(QPainter *painter,const QStyleOptionGraphicsItem *option,QWidget *par)
{
//   painter->setPen(Qt::red);
//   painter->drawRect(boundingRect());
	float trans;
	
	painter->setFont(font);
	painter->setPen(par_dev->settings->appearance.colors.at(NickName));
	if (par_dev->settings->appearance.display_mode==StackedHorizontally)
	{
		trans= par_dev->settings->text_height - get_nick_name_height();
		painter->drawText(rect_nick_name.translated(0.0,trans),Qt::AlignHCenter,par_dev->get_nick_name());
	}
	else
	{
		trans=0.0;
		painter->drawText(rect_nick_name,Qt::AlignHCenter,par_dev->get_nick_name());
	}
	
	if (par_dev->is_show_free_space() && (par_dev->get_state()==Mounted))
	{
		QString fs,us,ms;
		par_dev->get_free_space_text(fs,us,ms);
		painter->setFont(font_space);
		if (par_dev->settings->preferences.is_free_space)
		{			
			painter->setPen(par_dev->settings->appearance.colors.at(NickNameFreeSpace));
			painter->drawText(rect_free_space.translated(0.0,trans),Qt::AlignLeft,FREE_TEXT);
			painter->drawText(rect_free_space.translated(0.0,trans),Qt::AlignRight,fs);
		}
		if (par_dev->settings->preferences.is_used_space)
		{
			painter->setPen(par_dev->settings->appearance.colors.at(NickNameUsedSpace));
			painter->drawText(rect_used_space.translated(0.0,trans),Qt::AlignLeft,USED_TEXT);
			painter->drawText(rect_used_space.translated(0.0,trans),Qt::AlignRight,us);
		}
		if (par_dev->settings->preferences.is_max_space)
		{
			painter->setPen(par_dev->settings->appearance.colors.at(NickNameMaxSpace));
			painter->drawText(rect_max_space.translated(0.0,trans),Qt::AlignLeft,MAX_TEXT);
			painter->drawText(rect_max_space.translated(0.0,trans),Qt::AlignRight,ms);
		}
	}
}
	
void TextWidget::data_changed()
{
	manage_rects();
	update();
}

